/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.productionunit;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ProjectFactory;
import org.netbeans.spi.project.ProjectState;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Timon Veenstra
 */
@ServiceProvider(service = ProjectFactory.class)
public class ProductionUnitProjectFactory implements ProjectFactory {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.productionunit");
    public static final String ICON = "nl/cloudfarming/client/productionunit/grass.png";
    public static final String PRODUCTION_UNIT_FILE = "productionunit.bin";

    public ProductionUnitProjectFactory() {
        LOGGER.log(Level.FINEST, "ProductionUnitProjectFactory");
    }

    //Specifies when a project is a project, i.e.,
    //if the project directory "texts" is present:
    @Override
    public boolean isProject(FileObject projectDirectory) {
        LOGGER.log(Level.FINEST, "isProject asked for folder {0} concluded {1}", new Object[]{projectDirectory.getName(), projectDirectory.getFileObject(PRODUCTION_UNIT_FILE) != null});
        return projectDirectory.getFileObject(PRODUCTION_UNIT_FILE) != null;
    }

    //Specifies when the project will be opened, i.e.,
    //if the project exists:
    @Override
    public Project loadProject(FileObject dir, ProjectState state) throws IOException {
        LOGGER.log(Level.FINEST, "loading production unit project for dir {0}", dir.getPath());
        ProductionUnitProject project = null;
        if (isProject(dir)) {
            ProductionUnit productionUnit = null;
            FileObject productionUnitFile = dir.getFileObject(PRODUCTION_UNIT_FILE);
            if (productionUnitFile.getSize() > 0) {
                try (ObjectInputStream ois = new ObjectInputStream(productionUnitFile.getInputStream())) {
                    productionUnit = (ProductionUnit) ois.readObject();

                } catch (ClassNotFoundException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }

            if (productionUnit == null) {
                productionUnit = new ProductionUnit();
            }
            project = new ProductionUnitProject(dir, state, productionUnit);
        }
        return project;
    }

    @Override
    public void saveProject(final Project project) throws IOException, ClassCastException {
        FileObject projectRoot = project.getProjectDirectory();

        try (ObjectOutputStream oos = new ObjectOutputStream(projectRoot.createAndOpen(PRODUCTION_UNIT_FILE))) {
            oos.writeObject(((ProductionUnitProject) project).getProductionUnit());
        }


    }
}
